@@ -1,5 +1,6 @@
 package Class::AutoClass;
-our $VERSION = '1.49_02';
+our $VERSION = '1.56_01';
+$VERSION=eval $VERSION;         # I think this is the accepted idiom..
 
 use strict;
 use Carp;
@@ -57,16 +58,21 @@ sub new {
  for my $class (@$classes) {
    my $init_self = $class->can('_init_self');
    $self->$init_self( $class, $args ) if $init_self;
+   # NG 10-08-22: moved test for OVERRIDE to here to fix bug in which subsequent
+   #              calls to _init_self continue to operate on original $self !
+   $self=$self->{__OVERRIDE__} if $self->{__OVERRIDE__};
    #  $self->_init( $class, $args, $defaults, $default2code );
  }
 ################################################################################
 
    if($self->{__NULLIFY__}) {
    	return undef;
-   } elsif ($self->{__OVERRIDE__}) { # override self with the passed object
-      $self=$self->{__OVERRIDE__};
-      return $self;
-   } else {
+    # NG 10-08-22: moved test for OVERRIDE to here to fix bug in which subsequent
+    #              calls to _init_self continue to operate on original $self !
+    # } elsif ($self->{__OVERRIDE__}) { # override self with the passed object
+    #    $self=$self->{__OVERRIDE__};
+    #    return $self;
+      } else {
      return $self;
    }
 }
@@ -113,7 +119,14 @@ sub _init {
       grep {exists $args->{$fixed_attributes{$_}}} @$syn_list; # case 3
     # okay to set default!!
     my $value=$defaults->{$fixed_func};
-    $value=ref $value? dclone($value): $value; # deep copy refs so each instance has own copy
+    # NG 10-01-06: allow CODE and GLOB defaults. dclone can't copy these...
+    #              deep copy other refs so each instance has own copy
+    my $copy;
+    if (ref $value) {
+      $copy=eval{dclone($value)};
+      $value=$copy unless $@;	# use $copy unless dclone failed
+    }
+    # $value=ref $value? dclone($value): $value;
     $self->$fixed_func($value);
   }
 
@@ -521,6 +534,14 @@ sub declare {
   # add AutoDB::Object to @ISA if necessary
   unless ( grep /^Class::AutoDB::Object/, @{ $class . '::ISA' } ) {
     unshift @{ $class . '::ISA' }, 'Class::AutoDB::Object';
+    # NG 10-09-16: I thought it work work to push Object onto end of @ISA instead of 
+    #              unshifting it onto front to reduce impact of namespace pollution.
+    #              It does that okay, but introduces a new bug: oid generation and
+    #              all that is doen by Serialize which is a base class of Object. In
+    #              old implementation, that happened early; in new implementation, it
+    #              happens late.  Screws up a lot of things.:(
+    #              Back to the dawing boards...
+    # push @{ $class . '::ISA' }, 'Class::AutoDB::Object';
   }
   require 'Class/AutoDB/Object.pm';
   require 'Class/AutoDB.pm';    # AutoDB.pm is needed for calling auto_register
@@ -529,7 +550,12 @@ sub declare {
  # if (%autodb) {               # register after setting ANCESTORS
  if (UNIVERSAL::isa($class,'Class::AutoDB::Object')) {
    require 'Class/AutoDB.pm';    # AutoDB.pm is needed for calling auto_register
-   my $args = Hash::AutoHash::Args->new( %autodb, -class => $class ); # TODO - spec says %AUTODB=(1) should work
+   # NG 09-12-04: handle %AUTODB=0. (any single false value)
+   #              explicitly handle %AUTODB=1. previous version worked 'by luck' :)
+   confess "Illegal form of \%AUTODB. \%AUTODB=<false> reserved for future use"
+     if (scalar(keys %autodb)==1) && !(keys %autodb)[0];
+   delete $autodb{1};		# delete '1=>anything' if it exists 
+   my $args = Hash::AutoHash::Args->new( %autodb, -class => $class );
   Class::AutoDB::auto_register($args);
  }
  
@@ -794,7 +820,7 @@ Class::AutoClass - Create get and set methods and simplify object initialization
 
 =head1 VERSION
 
-Version 1.49_02
+Version 1.56
 
 =head1 SYNOPSIS
 
@@ -831,8 +857,8 @@ Version 1.49_02
     }
     return join(' ',$self->first_name,$self->last_name);
   }
-  ########################################
 
+  ########################################
   # code that uses class
   #
   use Person;
@@ -990,8 +1016,11 @@ One trap is that attribute-initialization occurs in arbitrary order.
 There is a temptation when writing 'set' methods to assume that
 attributes are initialized in the natural order that you would set
 them if you were writing the initialization code yourself.  I have
-been burned by this many times. This is only an issue for
-OTHER_ATTRIBUTES, of course.
+been burned by this many times. This is mainly an issue for
+OTHER_ATTRIBUTES. The issue also arises with SYNONYMS. If your code
+initializes both sides of a synonym with different values, it is
+undefined which value will stick. This can happen when your codes sets
+values explicitly or via DEFAULTS.
 
 The second trap involves "method resolution", ie, the way Perl chooses
 which sub to call when you invoke a method. Consider a class hierarchy
@@ -1101,7 +1130,7 @@ This class uses L<Hash::AutoHash::Args> to represent keyword=>value argument lis
 
 Nat Goodman, C<< <natg at shore.net> >>
 
-=head1 BUGS
+=head1 BUGS AND CAVEATS
 
 Please report any bugs or feature requests to C<bug-class-autoclass at
 rt.cpan.org>, or through the web interface at
